note

	description:

		"Short description of the class"

	library: "Gobo Eiffel ???? library"
	author: "Berend de Boer <berend@pobox.com>"
	copyright: "Copyright (c) 2011, Berend de Boer"
	license: "MIT License (see LICENSE)"
	date: "$Date$"
	revision: "$Revision$"


class

	PROJECT_DOCUMENT_PARSER


inherit

	XM_CALLBACKS_NULL
		redefine
			on_start_tag,
			on_content,
			on_end_tag
		end

	KL_IMPORTED_STRING_ROUTINES
		export
			{NONE} all
		end


create

	make_from_stream


feature {NONE} -- Initialization

	make_from_stream (a_stream: KI_CHARACTER_INPUT_STREAM)
		local
			parser: XM_EIFFEL_PARSER
		do
			create temp.make_empty
			create parser.make
			start_matchers := root_start_matcher
			parser.set_callbacks (Current)
			parser.parse_from_stream (a_stream)
			if not parser.is_correct then
				project := Void
			end
		end


feature -- Access

	project: PROJECT_DOCUMENT


feature {NONE} -- Events

	on_start_tag (a_namespace: STRING; a_prefix: STRING; a_local_part: STRING)
		local
			name: STRING
		do
			last_content := Void
			if a_namespace = Void then
				name := a_local_part
			else
				temp.wipe_out
				temp.append_string (a_namespace)
				temp.append_character (':')
				temp.append_string (a_local_part)
				name := temp
			end
			start_matchers.search (name)
			if start_matchers.found then
				start_matchers.found_item.call ([last_content])
			end
		end

	on_content (a_content: STRING)
			-- Text content.
		do
			if last_content = Void then
				last_content := a_content.twin
			else
				last_content.append_string (a_content)
			end
		end

	on_end_tag (a_namespace: STRING; a_prefix: STRING; a_local_part: STRING)
			-- End tag.
		local
			name: STRING
		do
			if end_matchers /= Void then
				if a_namespace = Void then
					name := a_local_part
				else
					temp.wipe_out
					temp.append_string (a_namespace)
					temp.append_character (':')
					temp.append_string (a_local_part)
					name := temp
				end
				print (name + ": " + last_content + "%N")
				end_matchers.search (name)
				if end_matchers.found then
					end_matchers.found_item.call ([last_content])
				end
			end
		end


feature {NONE} -- Fast switching

	root_start_matcher: DS_HASH_TABLE [PROCEDURE [ANY, TUPLE []], STRING]
		once
			create Result.make (1)
			Result.put (agent (s: STRING)
				do
					create project.make
					current_tuple := project.child_nodes
					start_matchers := project_start_matchers
					end_matchers := project_end_matchers
				end, once "project")
		ensure
			not_void: Result /= Void
		end

	project_start_matchers: DS_HASH_TABLE [PROCEDURE [ANY, TUPLE []], STRING]
		once
			create Result.make (1)
			Result.put (agent
				local
				t: like project.project_element
				l: DS_LINKED_LIST [like project.term_element]
				do
					t ?= current_tuple
					create l.make
					t.terms := l
					project.child_nodes.terms := l
				end, once "terms")
		ensure
			not_void: Result /= Void
		end

	project_end_matchers: DS_HASH_TABLE [PROCEDURE [ANY, TUPLE [STRING]], STRING]
		once
			create Result.make (3)
			Result.put (agent (s: STRING)
				local
				  t: like project.project_element
				do
					t ?= current_tuple
					t.title := s
				end, once "title")
			Result.put (agent (s: STRING)
				local
				  t: like project.project_element
				do
					t ?= current_tuple
					t.short_name := s
				end, once "short_name")
			Result.put (agent (s: STRING)
				local
				  t: like project.project_element
				do
					if s.is_integer then
						t ?= current_tuple
						t.recommended_major := s.to_integer
					end
				end, once "recommended_major")
		ensure
			not_void: Result /= Void
		end


feature {NONE} -- Implementation

	temp: STRING

	current_tuple: TUPLE

	last_content: STRING

	start_matchers: DS_HASH_TABLE [PROCEDURE [ANY, TUPLE []], STRING]
	end_matchers: DS_HASH_TABLE [PROCEDURE [ANY, TUPLE [STRING]], STRING]

invariant

	temp_not_void: temp /= Void

end
